@@ -1,5 +1,8 @@
 from django import forms
-from django.contrib.auth.models import User
+
+from django.contrib.auth import get_user_model
+
+User = get_user_model()
 
 
 class RegisterForm(forms.Form):
